DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
    }
}

INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.admin",
    "django.contrib.sites",
    "django.contrib.sitemaps",
    "django.contrib.staticfiles",
    "django.contrib.messages",
)

ROOT_URLCONF = "base_urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "OPTIONS": {
            "context_processors": [],
            "loaders": [],
        },
    },
]

MIDDLEWARE = []

SECRET_KEY = "a_key"



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

# Standard Library
import re

# Third party
from django.utils.translation import gettext as _
from django.utils.translation import pgettext

# Local application / specific library imports
from ..checks import custom_list


def importance():
    """Scripts with higher importance will be executed first.

    Returns:
        int -- Importance of the script.
    """
    return 1


def run(site):
    """Checks that h2 tags exist, and that at least one h2 contains at least one keyword.

    Arguments:
        site {Site} -- Structure containing a good amount of resources from the targeted webpage.
    """

    no_h2 = custom_list.CustomList(
        name=_("No h2 tag"),
        settings=_("at least one"),
        found=pgettext("masculin", "none"),
        description=_(
            "H2 tags are useful because they are explored by search engines and can help them understand the subject of your page."
        ),
    )

    enough_h2 = custom_list.CustomList(
        name=_("H2 tags were found"),
        settings=pgettext("feminin", "at least one"),
        description=no_h2.description,
    )

    no_keywords = custom_list.CustomList(
        name=_("No keyword in h2 tags"),
        settings=pgettext("masculin", "at least one"),
        found=pgettext("masculin", "none"),
        description=_(
            "Google uses h2 tags to better understand the subjects of your page."
        ),
    )

    enough_keywords = custom_list.CustomList(
        name=_("Keyword found in h2 tags"),
        settings=pgettext("masculin", "at least one"),
        found="",
        description=no_keywords.description,
    )

    h2 = site.soup.find_all("h2")
    if not h2:
        site.warnings.append(no_h2)
    else:
        enough_h2.found = len(h2)
        enough_h2.searched_in = [get_h2_text(t) for t in h2]
        site.success.append(enough_h2)

        occurrence = []
        h2_kw = []

        # for each h2...
        for single_h2 in h2:
            single_h2 = get_h2_text(single_h2).lower()
            # check if it contains at least 1 keyword
            for keyword in site.keywords:
                keyword_lower = keyword.lower()

                # standardize apostrophes
                keyword_lower = keyword_lower.replace("'", "’")
                single_h2 = single_h2.replace("'", "’")

                nb_occurrences = len(
                    re.findall(
                        r"(^| |\n|,|\.|!|\?)"
                        + keyword_lower.lower()
                        + r"($| |\n|,|\.|!|\?)",
                        single_h2,
                    )
                )
                occurrence.append(nb_occurrences)

                # add kw in found
                if nb_occurrences > 0:
                    # and add bold in found keywords
                    single_h2 = single_h2.replace(
                        keyword_lower, '<b class="good">' + keyword_lower + "</b>"
                    )
                    if enough_keywords.found != "":
                        enough_keywords.found += ", "
                    enough_keywords.found += keyword

            h2_kw.append(single_h2)
        # if no keyword is found in h2
        if not any(i > 0 for i in occurrence):
            no_keywords.searched_in = [t.text for t in h2]
            no_keywords.found = pgettext("masculin", "none")
            site.warnings.append(no_keywords)
        else:
            enough_keywords.searched_in = h2_kw
            site.success.append(enough_keywords)


def get_h2_text(h2):
    # h2 text can be content of alt tag in img
    if not h2.text and h2.find("img", {"alt": True}):
        return h2.find("img")["alt"]
    # of it can be the text in h2
    else:
        return h2.text



# -*- coding: utf-8 -*-

# Use ./launch_tests.sh to launch these tests.

from bs4 import BeautifulSoup
from django_check_seo.checks import site

html_content = """
<!doctype html>
<html>
    <head>
        <meta name="keywords" content="description,  title">
    </head>
</html>
"""


class init:
    def __init__(self):
        self.keywords = []
        self.problems = []
        self.warnings = []
        self.success = []

        self.soup = BeautifulSoup(html_content, features="lxml")
        self.full_url = "https://localhost/fake-url/title-of-the-page/"
        # populate class with data
        self.page_stats = site.Site(self.soup, self.full_url)


def test_keyword_importance():
    from django_check_seo.checks_list import check_keywords

    assert check_keywords.importance() == 5


def test_keyword_kw():
    from django_check_seo.checks_list import check_keywords

    site = init()

    check_keywords.run(site)

    for success in site.success:
        if success.name == "Keywords found in meta keywords field":
            assert success.name == "Keywords found in meta keywords field"
            assert success.settings == "at least one"
            assert success.found == 2
            assert success.searched_in == ["description", "title"]
            assert (
                success.description
                == "Django-check-seo uses the keywords in the meta keywords field to check all other tests related to the keywords. A series of problems and warnings are related to keywords, and will therefore systematically be activated if the keywords are not filled in."
            )


def test_keyword_nokw():
    from django_check_seo.checks_list import check_keywords

    site = init()
    site.soup.select('meta[name="keywords"]')[0]["content"] = ""

    check_keywords.run(site)

    for problem in site.problems:
        if problem.name == "No keywords in meta keywords field":
            assert problem.name == "No keywords in meta keywords field"
            assert problem.settings == "at least one"
            assert problem.found == "none"
            assert problem.searched_in == []
            assert (
                problem.description
                == "Django-check-seo uses the keywords in the meta keywords field to check all other tests related to the keywords. A series of problems and warnings are related to keywords, and will therefore systematically be activated if the keywords are not filled in."
            )



# Third party
from django.conf import settings

# define basic SEO settings, see
DJANGO_CHECK_SEO_SETTINGS = {
    "content_words_number": [300, 600],
    "internal_links": 1,
    "external_links": 1,
    "meta_title_length": [30, 60],
    "meta_description_length": [50, 160],
    "keywords_in_first_words": 50,
    "max_link_depth": 4,
    "max_url_length": 70,
}
# update settings redefined in projectname/settings.py
DJANGO_CHECK_SEO_SETTINGS.update(getattr(settings, "DJANGO_CHECK_SEO_SETTINGS", {}))

# define css selector to search content into (used for retrieving main content of the page)
DJANGO_CHECK_SEO_SEARCH_IN = {
    "type": "exclude",
    "selectors": ["header", ".cover-section", "#footer"],
}

#
DJANGO_CHECK_SEO_EXCLUDE_CONTENT = getattr(
    settings, "DJANGO_CHECK_SEO_EXCLUDE_CONTENT", ""
)
